/*
  SDL_Audiolib - An audio mixing, decoding and resampling library
  Copyright (C) 2013  Nikos Chantziaras

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
#ifndef AUDIORESAMPLER_H
#define AUDIORESAMPLER_H

#include "am_util.h"

namespace Aulib {

class AULIB_EXPORT AudioResampler {
public:
    friend class AudioResampler_priv;

    AudioResampler();
    virtual ~AudioResampler();

    void setDecoder(class AudioDecoder* decoder);

    int setSpec(unsigned dstRate, unsigned channels,
                unsigned chunkSize);

    unsigned currentRate() const;
    unsigned currentChannels() const;
    unsigned currentChunkSize() const;

    int resample(float dst[], int dstLen);

protected:
    virtual int doResampling(float dst[], const float src[],
                             int& dstLen, int& srcLen) = 0;

    virtual int adjustForOutputSpec(unsigned dstRate, unsigned srcRate,
                                    unsigned channels) = 0;

private:
    AudioResampler(const AudioResampler&);
    AudioResampler& operator =(const AudioResampler&);

    class AudioResampler_priv* d;
};

} // namespace Aulib

#endif
