/*
  SDL_Audiolib - An audio mixing, decoding and resampling library
  Copyright (C) 2013  Nikos Chantziaras

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
#ifndef AUDIOSTREAM_H
#define AUDIOSTREAM_H

#include "stream.h"

struct SDL_RWops;
struct SDL_AudioSpec;

namespace Aulib {

class AULIB_EXPORT AudioStream: public Stream {
public:
    AudioStream(const char* filename, class AudioDecoder* decoder,
                class AudioResampler* resampler);

    AudioStream(SDL_RWops* rwops, class AudioDecoder* decoder,
                class AudioResampler* resampler);

    ~AudioStream() override;

    int open() override;
    int play(unsigned iterations = 1) override;
    int stop() override;
    int rewind() override;
    int setVolume(float volume) override;
    float volume() const override;
    bool isPlaying() const override;
    Sint64 lengthMs() const override;

    static int init(int freq, int format, int channels, int bufferSize);
    static void quit();
    static const SDL_AudioSpec& spec();

private:
    friend class AudioStream_priv;
    class AudioStream_priv* d;
    AudioStream(const AudioStream&);
    AudioStream& operator =(const AudioStream&);
};

} // namespace Aulib

#endif
