#include <iostream>
#include <vector>
#include <string>
#include <cstdlib>
#include <SDL.h>

#include "audiostream.h"
#include "midistream.h"
#include "resamp_speex.h"
#include "resamp_src.h"
#include "resamp_sox.h"
#include "dec_vorbis.h"
#include "dec_mpg123.h"
#include "dec_modplug.h"
#include "dec_fluidsynth.h"
#include "dec_sndfile.h"

using namespace std;
using namespace Aulib;

#ifdef main
    #undef main
#endif


int main(int argc, char* argv[])
{
    if (SDL_Init(0) != 0) {
        cerr << "Unable to initialize SDL: " << SDL_GetError() << '\n';
        return 1;
    }

    int freq =
        //192000;
        //176400;
        //132300;
        //96000;
        //88200;
        //48000;
        44100;
        //32000;
        //22050;
        //11025;
        //8000;
        //2205;

    int format =
        //AUDIO_S16LSB;
        AUDIO_S32LSB;

    if (Aulib::AudioStream::init(freq, format, 2, 4096) < 0) {
        cerr << "Couldn't open audio: " << SDL_GetError() << '\n';
        return 1;
    }

    std::vector<Aulib::Stream*> streams;

    for (int i = 1; i < argc; ++i) {
        Aulib::Stream* s = nullptr;
        AudioResampler* resampler = nullptr;
        resampler = new AudioResamplerSRC;
        //resampler = new AudioResamplerSpeex;
        //resampler = new AudioResamplerSox;
        std::string str(argv[i]);

#ifdef AULIB_MPG123
        if (str.rfind(".mp3") != str.npos) {
            s = new Aulib::AudioStream(argv[i], new AudioDecoderMpg123, resampler);
        } else
#endif
#ifdef AULIB_VORBIS
        if (str.rfind(".ogg") != str.npos) {
            s = new Aulib::AudioStream(argv[i], new AudioDecoderVorbis, resampler);
        } else
#endif
#ifdef AULIB_FLUIDSYNTH
        if (str.rfind(".mid") != str.npos) {
            AudioDecoderFluidSynth* fsynth = new AudioDecoderFluidSynth;
            fsynth->loadSoundfont("/usr/share/timidity/soundfonts/8mb General MIDI.SF2");
            s = new Aulib::AudioStream(argv[i], fsynth, resampler);
        } else
#elif defined(AULIB_NATIVEMIDI)
        if (str.rfind(".mid") != str.npos) {
            s = new Aulib::MidiStream(argv[i]);
        } else
#endif
#ifdef AULIB_SNDFILE
        if (str.rfind(".wav") != str.npos or str.rfind(".flac") != str.npos) {
            s = new Aulib::AudioStream(argv[i], new AudioDecoderSndfile, resampler);
        } else
#endif
#ifdef AULIB_MODPLUG
        {
            s = new Aulib::AudioStream(argv[i], new AudioDecoderModPlug, resampler);
        }
#endif
        //SDL_RWops* rw = SDL_RWFromFile(argv[i], "rb");
        //Aulib::AudioStream* s = new Aulib::AudioStream(rw, Aulib::AudioStream::MediaTypeVorbis);
        //s->setVolume(1.f);
        if (s->play(1) != 0) {
            cerr << "Error playing stream\n";
        }
        streams.push_back(s);
        cout << "Length in ms of stream " << i << ": " << s->lengthMs() << '\n';
        //s->setFinishCallback([](Aulib::AudioStream&){cerr << "Finished\n";});
        //s->setLoopCallback([](Aulib::AudioStream&){cerr << "Loop\n";});
    }

    bool done;
    do {
        done = true;
        SDL_Delay(200);
        for (size_t i = 0; i < streams.size(); ++i) {
            if (streams[i]->isPlaying()) {
                done = false;
            }
        }
        //streams[0]->stop();
    } while (not done);
    //cin.ignore();
    //streams[0]->stop();
    //streams[0]->rewind();
    //cin.ignore();
    //streams[0]->play();
    //cin.ignore();

    for (size_t i = 0; i < streams.size(); ++i) {
        delete streams[i];
    }
    //void (*f)(void*, Uint8[], int) = sdlCallback;

    Aulib::AudioStream::quit();
    SDL_Quit();
}
