/*
  SDL_Audiolib - An audio mixing, decoding and resampling library
  Copyright (C) 2013  Nikos Chantziaras

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
#include "midistream.h"
#include <SDL_rwops.h>

using Aulib::MidiStream;


MidiStream::MidiStream(const char *filename)
{
    fRWops = SDL_RWFromFile(filename, "rb");
    fInit();
}


MidiStream::MidiStream(SDL_RWops *rwops)
    : fRWops(rwops)
{
    fInit();
}


MidiStream::~MidiStream()
{
    if (fIsPlaying) {
        native_midi_stop();
    }
    native_midi_freesong(fSong);
    SDL_RWclose(fRWops);
}


void
MidiStream::fInit()
{
    fIsOpen = false;
    fIsPlaying = false;
    fVolume = 1.f;
    fCurrentIteration = 0;
    fWantedIterations = 0;
}


int
MidiStream::open()
{
    if (fIsOpen) {
        return 0;
    }
    if (fRWops == nullptr) {
        return -1;
    }
    if (not native_midi_detect()) {
        return -1;
    }
    if ((fSong = native_midi_loadsong_RW(fRWops, false)) == nullptr ) {
        return -1;
    }
    fIsOpen = true;
    return 0;
}


int
MidiStream::play(unsigned iterations)
{
    if (not fIsOpen and open() < 0) {
        return -1;
    }
    fCurrentIteration = 0;
    fWantedIterations = iterations;
    native_midi_start(fSong, iterations);
    return 0;
}


int
MidiStream::stop()
{
    if (not fIsOpen and open() < 0) {
        return -1;
    }
    if (fIsPlaying) {
        native_midi_stop();
    }
    return 0;
}


int
MidiStream::rewind()
{
    if (not fIsOpen and open() < 0) {
        return -1;
    }
    if (not fIsPlaying) {
        native_midi_stop();
        native_midi_start(fSong, fWantedIterations - fCurrentIteration);
    }
    return 0;
}


int
MidiStream::setVolume(float volume)
{
    if (volume < 0.f) {
        volume = 0.f;
    }
    fVolume = volume;
    return 0;
}


float
MidiStream::volume() const
{
    return fVolume;
}
