/*
  SDL_Audiolib - An audio mixing, decoding and resampling library
  Copyright (C) 2013  Nikos Chantziaras

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
#ifndef MIDISTREAM_H
#define MIDISTREAM_H

#include "stream.h"
extern "C" {
#include "native_midi.h"
}

struct SDL_RWops;

namespace Aulib {

class AULIB_EXPORT MidiStream: public Stream {
public:
    MidiStream(const char* filename);
    MidiStream(SDL_RWops* rwops);
    ~MidiStream() override;

    int open() override;
    int play(unsigned iterations = 1) override;
    int stop() override;
    int rewind() override;
    int setVolume(float volume) override;
    float volume() const override;

private:
    bool fIsOpen;
    SDL_RWops* fRWops;
    NativeMidiSong* fSong;
    bool fIsPlaying;
    float fVolume;
    unsigned fCurrentIteration;
    unsigned fWantedIterations;

    MidiStream(const MidiStream&);
    MidiStream& operator =(const MidiStream&);

    void fInit();
};

} // namespace Aulib

#endif // MIDISTREAM_H
