/*
  SDL_Audiolib - An audio mixing, decoding and resampling library
  Copyright (C) 2013  Nikos Chantziaras

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
#include "resamp_sox.h"

#include <iostream>
#include <cstring>
#include <soxr.h>

using Aulib::AudioResamplerSox;


class Aulib::AudioResamplerSox_priv {
    friend class AudioResamplerSox;

    AudioResamplerSox_priv();
    ~AudioResamplerSox_priv();

    soxr_t fResampler;
};


Aulib::AudioResamplerSox_priv::AudioResamplerSox_priv()
    : fResampler(nullptr)
{
}


Aulib::AudioResamplerSox_priv::~AudioResamplerSox_priv()
{
    if (fResampler) {
        soxr_delete(fResampler);
    }
}


AudioResamplerSox::AudioResamplerSox()
    : d(new AudioResamplerSox_priv)
{
}


AudioResamplerSox::~AudioResamplerSox()
{
    delete d;
}


int
AudioResamplerSox::doResampling(float dst[], const float src[], int& dstLen,
                                int& srcLen)
{
    if (d->fResampler == nullptr) {
        dstLen = srcLen = 0;
        return 0;
    }
    size_t dstDone, srcDone, channels = currentChannels();
    soxr_error_t error;
    error = soxr_process(d->fResampler, src, srcLen / channels, &srcDone,
                         dst, dstLen / channels, &dstDone);
    if (error) {
        std::cerr << "soxr_process() error: " << error << '\n';
    }
    dstLen = dstDone * channels;
    srcLen = srcDone * channels;
    return dstLen;
}


int
AudioResamplerSox::adjustForOutputSpec(unsigned dstRate, unsigned srcRate,
                                       unsigned channels)
{
    if (d->fResampler) {
        soxr_delete(d->fResampler);
    }

    soxr_io_spec_t spec;
    memset(&spec, 0, sizeof(spec));
    spec.itype = spec.otype = SOXR_FLOAT32_I;
    spec.scale = 1.0;

    //soxr_quality_spec_t quality = soxr_quality_spec()

    soxr_error_t error;
    d->fResampler = soxr_create(srcRate, dstRate, channels, &error, &spec,
                                nullptr, nullptr);
    if (error) {
        d->fResampler = nullptr;
        return -1;
    }
    return 0;
}
