/*
  SDL_Audiolib - An audio mixing, decoding and resampling library
  Copyright (C) 2013  Nikos Chantziaras

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
#include "resamp_speex.h"
#include "am_util.h"

#include <iostream>
#include "speex_resampler.h"
#include "audiodecoder.h"

using namespace std;
using Aulib::AudioResamplerSpeex;


class Aulib::AudioResamplerSpeex_priv {
    friend class AudioResamplerSpeex;

    AudioResamplerSpeex_priv();
    ~AudioResamplerSpeex_priv();

    SpeexResamplerState* fResampler;
};


Aulib::AudioResamplerSpeex_priv::AudioResamplerSpeex_priv()
    : fResampler(nullptr)
{
}


Aulib::AudioResamplerSpeex_priv::~AudioResamplerSpeex_priv()
{
    if (fResampler) {
        speex_resampler_destroy(fResampler);
    }
}


AudioResamplerSpeex::AudioResamplerSpeex()
    : d(new AudioResamplerSpeex_priv)
{
}


AudioResamplerSpeex::~AudioResamplerSpeex()
{
    delete d;
}


int
AudioResamplerSpeex::doResampling(float dst[], const float src[], int& dstLen,
                                  int& srcLen)
{
    if (d->fResampler == nullptr) {
        dstLen = srcLen = 0;
        return 0;
    }

    unsigned channels = currentChannels();
    spx_uint32_t spxInLen = srcLen / channels;
    spx_uint32_t spxOutLen = dstLen / channels;
    if (spxInLen == 0 or spxOutLen == 0) {
        dstLen = srcLen = 0;
        return 0;
    }
    //cerr << "(before) spxInLen: " << spxInLen << " spxOutLen: " << spxOutLen << '\n';
    speex_resampler_process_interleaved_float(d->fResampler, src, &spxInLen, dst,
                                              &spxOutLen);
    //cerr << "(after) spxInLen: " << spxInLen << " spxOutLen: " << spxOutLen << '\n';
    dstLen = spxOutLen * channels;
    srcLen = spxInLen * channels;
    return dstLen;
}


int
AudioResamplerSpeex::adjustForOutputSpec(unsigned dstRate, unsigned srcRate,
                                         unsigned channels)
{
    if (d->fResampler) {
        speex_resampler_destroy(d->fResampler);
    }
    int err;
    d->fResampler = speex_resampler_init(channels, srcRate, dstRate, 10, &err);
    if (err) {
        d->fResampler = nullptr;
        return -1;
    }
    return 0;
}
