/*
  SDL_Audiolib - An audio mixing, decoding and resampling library
  Copyright (C) 2013  Nikos Chantziaras

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
#ifndef STREAM_H
#define STREAM_H

#include "am_util.h"

#include <boost/function.hpp>
#include <SDL_stdinc.h>

namespace Aulib {

/*!
 * \brief Abstract base class for audio streams.
 *
 * The Stream class provides a common interface for audio streams. The class
 * is abstract and most functionality is implemented by the AudioStream and
 * MidiStream subclasses.
 */
class AULIB_EXPORT Stream {
public:
    //!
    typedef boost::function<void (Stream&)> FinishCallback;
    typedef FinishCallback LoopCallback;

    Stream();
    virtual ~Stream();

    virtual int open() = 0;
    virtual int play(unsigned iterations = 1) = 0;
    virtual int stop() = 0;
    virtual int rewind() = 0;
    virtual int setVolume(float volume) = 0;
    virtual float volume() const = 0;
    virtual bool isPlaying() const = 0;
    virtual Sint64 lengthMs() const = 0;

    void setFinishCallback(FinishCallback func);
    void unsetFinishCallback();
    void setLoopCallback(LoopCallback func);
    void unsetLoopCallback();

protected:
    void invokeFinishCallback();
    void invokeLoopCallback();

private:
    class Stream_priv* d;
};

}

#endif
